@@ -26,7 +26,7 @@
 #include <gtk/gtk.h>
 #include "NvCtrlAttributes.h"
 
-#include "ctkimage.h"
+#include "ctkbanner.h"
 
 #include "ctkxvideo.h"
 #include "ctkscale.h"
